#ifndef CDAQ_UTILS_H
#define CDAQ_UTILS_H

#include <QObject>
#include <QStringList>
#include <QString>
#include <QList>
#include <QListData>


#include "csql_utils.h"
#include "cstatistics_utils.h"
#include "cgps_utils.h"


class cDaq_utils : public QObject
{
    Q_OBJECT
public:
    explicit cDaq_utils(QObject *parent = 0);

signals:

    void daq_ready(QString msg);


public slots:

    void receive_data(QStringList strlist);


public:

    QString get_dataRequest_string();

    void parse_message(QStringList strlist);
    void extract_data(QStringList strlist);
    bool id_is_in_list(int id);

    void load_engr_coefs();  //load engr coefs from data-base
    //assume they're correct when program is run; i.e. they don't change during running of exe

    double calc_weight(double val);
    double calc_pressure(double val);

    double slope_press = 499.67;  //default vals for now (till we implement reading sqlite database for the parms
    double intercept_press = 0.8567;

    double slope_weight = 4.1724;   //USE FOR INITIAL ESTIMATE TO SEE WHICH EQ TO USE
    double intercept_weight = -2467.1;

    double weight_threshold_01 = 800.0;
    double weight_threshold_12 = 1600.0;


    double slope_01_weight = 4.1724;   //USE FOR WEIGHTS BELOW weight_threshold_01
    double intercept_01_weight = -2467.1;

    double slope_12_weight = 4.1724;   //USE FOR WEIGHTS between weight_threshold_01 & weight_threshold_12
    double intercept_12_weight = -2467.1;

    double slope_23_weight = 4.1724;   //USE FOR WEIGHTS ABOVE weight_threshold_12
    double intercept_23_weight = -2467.1;


    bool bTareMode = false;
    double tare_weight = 0.0;


    int pwm1_timeOut = 30;
    int pwm1_firstStage_dutyCycle = 95;
    int pwm1_secondStage_dutyCycle = 95;


    double stability = 0.0;
    double stability_threshold_weight = 25.0;

    int data_below_to_ignore = 19;  //ignore first 10 dataPoints
    int data_above_to_ignore = 59;  //skip 60 and above


    int loadNum = 0;
    double weight = 0.0;
    double pressure = 0.0;
    double yield = 0.0;


    QString sLoadNum = "1";
    QString sWeight = "0.0";
    QString sPressure = "0.0";
    QString sFieldID = "f20";
    QString sYield = "0";
    QString sStability = "0";
    QString sWindCondition = "NORMAL";

    QString sTareWeight = "0.0";


    QList<double> raw_data;
    QList<int> raw_data_ids;

    void test_daq();

    QString weighOpRequest = "$,3,27,40,30,70,31,40";
    QString daqAliveRequest = "#1";

    cSQL_Utils *csql;

    cGPS_utils *cgps;

private:

    QString test_msg1 = "$1,232,2,585,3,1,4,0";
    QString test_msg2 = "$1,232,2,585,3,1,4,0";

    cStatistics_utils *cStats;


};


#endif // CDAQ_UTILS_H
