
#include "csql_utils.h"

#include "csql_utils.h"\

#include <QCoreApplication>
#include <QString>
#include <QStringList>
#include <iostream>
#include <QDir>

#include <QStandardPaths>

#include <QMessageBox>


#include <QtSql/QSqlDatabase>
#include <QtSql>
#include <QtSql/QSqlField>

//put it here for testing purposes; but can't leave it here as we can only have one open
//QSqlDatabase db2;

void test_basic_sqlite()
{
    QSqlDatabase db2;

    //QSqlDatabase db = QSqlDatabase::addDatabase( "QSQLITE" );
    db2 = QSqlDatabase::addDatabase( "QSQLITE" );

    db2.setDatabaseName( "./testdatabase.db" );

    if( !db2.open() )
    {
      qDebug() << db2.lastError();
      qFatal( "Failed to connect." );
    }

    qDebug( "Connected!" );

      QSqlQuery qry;

      qry.prepare( "CREATE TABLE IF NOT EXISTS names (id INTEGER UNIQUE PRIMARY KEY, firstname VARCHAR(30), lastname VARCHAR(30))" );
      if( !qry.exec() )
        qDebug() << qry.lastError();
      else
        qDebug() << "Table created!";

      qry.prepare( "INSERT INTO names (id, firstname, lastname) VALUES (1, 'John', 'Doe')" );
      if( !qry.exec() )
        qDebug() << qry.lastError();
      else
        qDebug( "Inserted!" );


      qry.prepare( "SELECT * FROM names" );
        if( !qry.exec() )
          qDebug() << qry.lastError();
        else
        {
          qDebug( "Selected!" );

          QSqlRecord rec = qry.record();

          int cols = rec.count();

          for( int c=0; c<cols; c++ )
            qDebug() << QString( "Column %1: %2" ).arg( c ).arg( rec.fieldName(c) );

          for( int r=0; qry.next(); r++ )
            for( int c=0; c<cols; c++ )
              qDebug() << QString( "Row %1, %2: %3" ).arg( r ).arg( rec.fieldName(c) ).arg( qry.value(c).toString() );
        }

    db2.close();

}

//general test of functions
void test_csql_utils()
{
    QString dbName1 = "test_db1";
    QString dbName2 = "test_db2";

    QString dbPath2 = "/usr/lib/cgi-bin/test_db3.sqlite";


    QString tableName1 = "personnel";
    QString tableName2 = "executives";

    QStringList tableColumnNames1;
    QStringList tableColumnNames2;

    QStringList tableTypes1;
    QStringList tableTypes2;


    tableColumnNames1 << "names" << "phone" << "address" << "age" << "salary";
    tableColumnNames2 << "names" << "title" ;

    tableTypes1 << "TEXT" << "TEXT" << "TEXT" << "TEXT" << "TEXT";
    tableTypes2 << "TEXT" << "TEXT";


    cSQL_Utils csql;

    csql.deleteDB();

    //use default path
    csql.create_sqlite_dataBase();


    bool ok = csql.testDB_tableExists(tableName1);

    if(!ok)
    {
        bool success = csql.createTable(tableName1, tableColumnNames1, tableTypes1);

        if(!success)
        {
            //error creating table, likely miss-match in # of columns for names and types
            QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in test_csql_utils()";
//            QMessageBox::warning(this,"Thread Error", sErMsg );

        }

        ok = csql.testDB_tableExists(tableName1);


    }

    QStringList colNameList1 = csql.getDB_tableColumnNames(tableName1);
    QStringList colTypeList1 = csql.getDB_tableTypeNames(tableName1);


    ok = csql.testDB_tableExists(tableName2);

    if(!ok)
    {
        bool success = csql.createTable(tableName2, tableColumnNames2, tableTypes2);

        if(!success)
        {
            //error creating table, likely miss-match in # of columns for names and types

        }

        ok = csql.testDB_tableExists(tableName1);


    }

    QStringList colNameList2 = csql.getDB_tableColumnNames(tableName2);
    QStringList colTypeList2 = csql.getDB_tableTypeNames(tableName2);




    //now add some data
    QStringList columnValues;
    columnValues << "George" << "(806)123-456" << "86 CR 2000 Lubb TX" << "45" << "$126000";

    int rowID = csql.insertROW_into_table(tableName1, tableColumnNames1, columnValues);

    columnValues.clear();
    columnValues << "Jeff" << "(806)123-456" << "86 CR 5000 Lubb TX" << "65" << "$186000";

    rowID = csql.insertROW_into_table(tableName1, tableColumnNames1, columnValues);


    //now read them back out
    int cnt = csql.getLast_rowID_in_table(tableName1);

    QStringList sRow;
    for(int i=1;i<=cnt;i++)
    {
        rowID = i;
        sRow = csql.getROW_table(tableName1, rowID);
    }

    //delete row (note: won't re-order data-base, row with id=2 is still id=2; you'll just skip id=1)
    csql.deleteROW_in_table(tableName1, 1);

    //now read them back out
    cnt = csql.getLast_rowID_in_table(tableName1);

    sRow.clear();
    for(int i=1;i<=cnt;i++)
    {
        rowID = i;
        sRow = csql.getROW_table(tableName1, rowID);
    }



    //add a 2nd dataBase with ATTACH
    csql.attach_sqlite_dataBase(dbName2,dbPath2);

}


//factory installed on DataLogger; so only 1 dB required/wanted per unit
//all setups will take place on each dataLogger, so master doesn't mess with it (except its own setup)
//will need some functions to all for obtaining file-dumps so master can collect all data-togeather into 1 file for final download for user

void create_dataLogger_DB()
{

    QString tableName1 = "ModelSpecs";
    QStringList tableColumnNames1;
    QStringList tableTypes1;


    tableColumnNames1 << "DLog_Model" << "SerialNum" << "DateOfManuf" << "Warranty" << "Options" << "Max_AdcSpeed_Hz" << "Adc_bits" << "Adc_MaxVolts" << "Pwr_MinVolts" << "Pwr_MaxVolts";
    tableTypes1       << "TEXT"       << "TEXT"      << "TEXT"        << "TEXT"     << "TEXT"    << "TEXT"            << "TEXT"     << "TEXT"         << "TEXT"         << "TEXT"  ;


    cSQL_Utils csql;

    //START WITH CLEAN SLATE FOR DataLogger DataBase
    csql.deleteDB();


    csql.create_sqlite_dataBase();  //creates dataBase in Documents folder


    bool ok = csql.testDB_tableExists(tableName1);

    if(!ok)
    {
        bool success = csql.createTable(tableName1, tableColumnNames1, tableTypes1);

        if(!success)
        {
            //error creating table, likely miss-match in # of columns for names and types
            QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in create_dataLogger_DB()";
//            QMessageBox::warning(this,"Thread Error", sErMsg );

        }

        ok = csql.testDB_tableExists(tableName1);


    }

    //check it by reading it back
    QStringList colNameList1 = csql.getDB_tableColumnNames(tableName1);
    QStringList colTypeList1 = csql.getDB_tableTypeNames(tableName1);




//build dataBase Table for model functionality; i.e. what channels there are what they can do; sw, dig out, dig_in, adc, padc, pwm ect

    QString tableName2 = "ModelChannels";
    QStringList tableColumnNames2;
    QStringList tableTypes2;


    tableColumnNames2 << "Channel_Op_Name" << "Func1" << "Func2" << "Func3" << "Func4" << "Func5" << "Func6" << "Func7";
    tableTypes2       << "TEXT"     << "TEXT"  << "TEXT"  << "TEXT"  << "TEXT"  << "TEXT"  << "TEXT"  << "TEXT";



    ok = csql.testDB_tableExists(tableName2);

    if(!ok)
    {
        bool success = csql.createTable(tableName2, tableColumnNames2, tableTypes2);

        if(!success)
        {
            //error creating table, likely miss-match in # of columns for names and types
            QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in create_dataLogger_DB()";
//            QMessageBox::warning(this,"Thread Error", sErMsg );

        }

        ok = csql.testDB_tableExists(tableName2);


    }

    //check it by reading it back
    QStringList colNameList2 = csql.getDB_tableColumnNames(tableName2);
    QStringList colTypeList2 = csql.getDB_tableTypeNames(tableName2);





    //build dataBase Table for farm INFO;i.e. user's farm, field,


        QString tableName4 = "FarmINFO";
        QStringList tableColumnNames4;
        QStringList tableTypes4;


        tableColumnNames4 << "FARM" << "CLIENT" << "FIELD" << "NUM_ROWS" << "ROW_WIDTH";
        tableTypes4       << "TEXT" << "TEXT"   << "TEXT"  << "TEXT"     << "TEXT" ;



        ok = csql.testDB_tableExists(tableName4);

        if(!ok)
        {
            bool success = csql.createTable(tableName4, tableColumnNames4, tableTypes4);

            if(!success)
            {
                //error creating table, likely miss-match in # of columns for names and types
                QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in create_dataLogger_DB()";
    //            QMessageBox::warning(this,"Thread Error", sErMsg );

            }

            ok = csql.testDB_tableExists(tableName4);


        }

        //check it by reading it back
        QStringList colNameList4 = csql.getDB_tableColumnNames(tableName4);
        QStringList colTypeList4 = csql.getDB_tableTypeNames(tableName4);










        //build dataBase Table for farm INFO;i.e. user's farm, field,


            QString tableName5 = "ENGR_PARMS";
            QStringList tableColumnNames5;
            QStringList tableTypes5;


            tableColumnNames5 << "PARM_ID" << "VALUE";
            tableTypes5       << "TEXT"    << "TEXT" ;



            ok = csql.testDB_tableExists(tableName5);

            if(!ok)
            {
                bool success = csql.createTable(tableName5, tableColumnNames5, tableTypes5);

                if(!success)
                {
                    //error creating table, likely miss-match in # of columns for names and types
                    QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in create_dataLogger_DB()";
        //            QMessageBox::warning(this,"Thread Error", sErMsg );

                }

                ok = csql.testDB_tableExists(tableName5);


            }

            //check it by reading it back
            QStringList colNameList5 = csql.getDB_tableColumnNames(tableName5);
            QStringList colTypeList5 = csql.getDB_tableTypeNames(tableName5);


            QStringList colVals;
            colVals << "NUM_ROWS" <<  "6" ;

            int rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);


            colVals.clear();
            colVals << "ROW_WIDTH_IN" <<  "40" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "SLOPE_PRESSURE" <<  "499.67" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "INTERCEPT_PRESSURE" <<  "0.8567" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "SLOPE_WEIGHT" <<  "4.1724" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "INTERCEPT_WEIGHT" <<  "-2467.1" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);


            colVals.clear();
            colVals << "PWM1_TIMEOUT" <<  "30" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "DUTY_CYCLE_PWM1_FIRST_STAGE" <<  "95" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "DUTY_CYCLE_PWM1_SECOND_STAGE" <<  "90" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "WEIGH_OP_REQUEST" <<  "$1,10,2,10,3,4,5,4" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);

            colVals.clear();
            colVals << "DAQ_ALIVE_REQUEST" <<  "#1" ;
            rowID = csql.insertROW_into_table(tableName5, tableColumnNames5, colVals);




//build dataBase table for USER's Operational Setup ; i.e. what channels we're using and what func there providing; sw, dig out, dig_in, adc, padc, pwm ect

        QString tableName3 = "DAQ_Channels_CONFIG";
        QStringList tableColumnNames3;
        QStringList tableTypes3;


        tableColumnNames3 << "Channel_Op_Name" << "UserFreindlyName" << "Units" << "Function" << "UpdateRate" << "Slope" << "Intercept" << "DutyCycle" << "PWM_Freq" << "Initial_State";
        tableTypes3       << "TEXT"     << "TEXT" << "TEXT" << "TEXT"     << "TEXT"       << "TEXT"  << "TEXT"      << "TEXT"      << "TEXT"     << "TEXT";


        ok = csql.testDB_tableExists(tableName3);

        if(!ok)
        {
            bool success = csql.createTable(tableName3, tableColumnNames3, tableTypes3);

            if(!success)
            {
                //error creating table, likely miss-match in # of columns for names and types
                QString sErMsg = "DB Creation Error, error creating table, likely miss-match in # of columns for names and types, in create_dataLogger_DB()";
//               QMessageBox::warning(this,"Thread Error", sErMsg );

            }

            ok = csql.testDB_tableExists(tableName3);


        }

        //check it by reading it back
        QStringList colNameList3 = csql.getDB_tableColumnNames(tableName3);
        QStringList colTypeList3 = csql.getDB_tableTypeNames(tableName3);









//now add data for Specs table
    QStringList columnValues;
    //tableColumnNames1 << "DLog_Model" << "SerialNum" << "DateOfManuf" << "Warranty" << "Options" << "Max_AdcSpeed_Hz" << "Adc_bits" << "Adc_MaxVolts"          << "Pwr_MaxVolts";
    columnValues        << "DLog_5000" <<  "S20234"    << "1_24_15"     << "360 Days" << "ALL"     << "10 Hz"            << "20 bits" << "5 Volts Max Adc Input" << "Power Max Voltage = 30 Volts";

    rowID = csql.insertROW_into_table(tableName1, tableColumnNames1, columnValues);


QStringList tableColNames;
//tableColumnNames2 << "Channel_Op_Name" << "Func1" << "Func2" << "Func3" << "Func4" << "Func5" << "Func6" << "Func7";

//Load DataLogger Channels Table
    columnValues.clear();
    tableColNames << "Channel_Op_Name" << "Func1";
    columnValues        << "PADC1"    << "PADC";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "PADC2"    << "PADC";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "PWM1"    << "PWM";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "PWM2"    << "PWM";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "PWM3"    << "PWM";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "PWM4"    << "PWM";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "SW1"      << "DigIn";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "SW2"      << "DigIn";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "GPS"      << "ttyS0";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);

    columnValues.clear();
    columnValues        << "DAQ"      << "ttyS1";
    rowID = csql.insertROW_into_table(tableName2, tableColNames, columnValues);






    //now read them back out
    int cnt = csql.getLast_rowID_in_table(tableName2);

    QStringList sRow;
    for(int i=1;i<=cnt;i++)
    {
        rowID = i;
        sRow = csql.getROW_table(tableName2, rowID);
    }




//to create DATA TABLE
//CREATE TABLE DATA (id integer primary key, LOAD_NUM TEXT, CLIENT TEXT, FARM TEXT, FIELD TEXT, NUM_ROWS TEXT, ROW_WIDTH TEXT, YIELD TEXT, WEIGHT TEXT, AREA TEXT, DISTANCE TEXT, STABILITY TEXT, PRESSURE TEXT, RAW TEXT, START_LAT TEXT, START_LON TEXT, STOP_LAT TEXT, STOP_LON TEXT)



    csql.closeDB();



    //last table is for user to setup, so we'll leave blank
    //use clearDB_table() for if/when a user wants to clear it and start over; else use UPDATE for various rows...

    //last table is missing as it'll be built from actual channels used for collecting data
    //note it'll be built similarly to table3




}

bool test_dataLogger_DB()
{
    cSQL_Utils csql;

    //START WITH CLEAN SLATE FOR DataLogger DataBase
    csql.open_sqlite_dataBase();
    //csql.open_sqlite_dataBase(dbPath);

    //csql.deleteDB();

    QString tableName = "ModelChannels";

    bool ok = csql.testDB_tableExists(tableName);

    return ok;

}


void open_dataLogger_DB()
{
    //QString dbName1 = "DataLogger_Specs";

    //QString dbPath = QDir::currentPath() + "dlogSpecs.sqlite";
//    QString dbPath = "/usr/lib/cgi-bin/dlogSpecs.sqlite";
    QString dbPath = "/home/ubuntu14/cgi/dlogSpecs3.sqlite";

    QString dbName = "MASTER_DATALOG_DB";


    QString tableName1 = "ModelSpecs";
    QStringList tableColumnNames1;
    QStringList tableTypes1;


    tableColumnNames1 << "DLog_Model" << "SerialNum" << "DateOfManuf" << "Warranty" << "Options" << "Max_AdcSpeed_Hz" << "Adc_bits" << "Adc_MaxVolts" << "Pwr_MaxVolts";
    tableTypes1       << "TEXT"       << "TEXT"      << "TEXT"        << "TEXT"     << "TEXT"    << "TEXT"            << "TEXT"     << "TEXT"         << "TEXT"  ;


    cSQL_Utils csql;

    //START WITH CLEAN SLATE FOR DataLogger DataBase
    csql.open_sqlite_dataBase();
    //csql.open_sqlite_dataBase(dbPath);

    //csql.deleteDB();

    QString tableName2 = "ModelChannels";

    bool ok = csql.testDB_tableExists(tableName2);


    if(ok)
    {
        //now read them back out
        int cnt = csql.getLast_rowID_in_table(tableName2);

        QStringList sRow;
        for(int i=1;i<=cnt;i++)
        {
            int rowID = i;
            sRow = csql.getROW_table(tableName2, rowID);
        }
    }


    csql.closeDB();

}

